#Exploración y análisis lluvia mensual y anual
#http://sthda.com/english/wiki/ggplot2-essentials
#https://ggplot2.tidyverse.org/reference/ggplot.html

rm(list=ls()) # borrar variables al inicio

library(ggplot2)
library(moments)#skewness
library(zoo)#series temporales

###########################################################################
##1. abrir archivo .csv
file0 = read.csv("costa.csv",skip = 0,header=TRUE, sep=",")

###########################################################################
##2. transformar datos costa
file1 = NA
for(i in 1:nrow(file0)){
  file = data.frame(rep(file0[i,1],12),colnames(file0[i,-1]),as.numeric(file0[i,-1]))
  colnames(file) = c("year","month","rain")
  file1 = rbind.data.frame(file1, file)
}
costa = file1[-1,]

###########################################################################
##3. Obtener la estadística de las series: media , desviación estándar, coeficiente de asimetría.
print(paste("Costa: avg= ",round(mean(costa$rain, na.rm = T),1),
            ", sd= ",round(sd(costa$rain, na.rm = T),1),
            ", skewness= ",round(skewness(costa$rain, na.rm = T),1) ))

print(summary(costa))
hist(costa$rain) # ¿distribución normal?
hist(costa$rain, prob=TRUE) # con probabilidad
lines(density(costa$rain), col = "red", lwd = 2)

###########################################################################
##4.Graficar la serie temporal 
#Transformar data.frame a serie temporal
ts_costa = as.POSIXct(paste(costa$year, costa$month, "01",sep="-"), format = "%Y-%b-%d") # crear serie temporal
costa2 = cbind.data.frame(costa,ts_costa)
colnames(costa2) = c("year","month","rain","date")
z_costa = zoo(costa2$rain,costa2$date)

#plot serie temporales
plot_costa_mes = ggplot(costa2, aes(x=date, y=rain))+
  geom_step()+
  theme_bw() +
  labs (title="Lluvia  mensual costa 1947-2012", x= NULL, y="Lluvia mensual [mm/mes]")+
  theme(plot.title = element_text(size = 20),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))
print(plot_costa_mes)

png(file='plot1_serie_temporal.png', height=400, width=800)
plot_costa_mes
dev.off() 

###########################################################################
##5.Precipitación media mensual [mm/mes]
#precipitación media mensual
avg_mes = round(colMeans(file0[,-1], na.rm = T) ,1)
print(avg_mes)

#plot precipitación media mensual
costa2$month = factor(costa2$month, levels = c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"))
rain_month = ggplot(costa2, aes(x=month, y=rain))+
  geom_boxplot()+
  theme_bw() +
  labs (title="Lluvia media mensual Costa 1947-2012", x= NULL, y="Lluvia [mm/mes]")+
  theme(plot.title = element_text(size = 20),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))+
  annotate(geom="text", x=9, y=1200, label="Sebastián Páez-Bimos",color="blue", size=5)

png(file='plot2_lluvia_media_mensual.png', height=400, width=800, res = 100)
rain_month
dev.off()  

###########################################################################
##6.Agregación precipitación mensual a anual
rain_year <- aggregate(rain ~ year, costa2, sum)
print(rain_year)
print(summary(rain_year))
hist(rain_year$rain) # ¿distribución normal?
hist(rain_year$rain, prob=TRUE) # ¿distribución normal?
lines(density(rain_year$rain), col = "red", lwd = 2)


###########################################################################
##7.Guardar y cargar variaables .RData
#guradar datos
save(costa2, file = "file_costa_mes.RData")
save(rain_year, file = "file_costa_anual.RData")

rm(list=ls()) # borrar variables 

#cargar variables
load("file_costa_mes.RData") #costa2
load("file_costa_anual.RData") #rain_year

###########################################################################
#