
#Relleno y curva doble masas

rm(list=ls()) # borrar variables al inicio

#install.packages("moments")        # Install moments package
#install.packages("zoo")            # Install zoo package
#install.packages("reshape")        # Install reshape package
library(moments)
library(zoo)
library(reshape)
library(ggplot2)

###########################################################################
##1. abrir archivos .csv y transformarlos
###########################################################################
costa1 = read.csv("costa1.csv",skip = 0,header=TRUE, sep=",")
costa2 = read.csv("costa2.csv",skip = 0,header=TRUE, sep=",")

##transform data costa
file1 = NA
for(i in 1:nrow(costa1)){
  file = data.frame(rep(costa1[i,1],12),colnames(costa1[i,-1]),as.numeric(costa1[i,-1]))
  colnames(file) = c("year","month","rain")
  file1 = rbind.data.frame(file1, file)
}
costa1a = file1[-1,]

##transform data costa2
file1 = NA
for(i in 1:nrow(costa2)){
  file = data.frame(rep(costa2[i,1],12),colnames(costa2[i,-1]),as.numeric(costa2[i,-1]))
  colnames(file) = c("year","month","rain")
  file1 = rbind.data.frame(file1, file)
}
costa2a = file1[-1,]

###########################################################################
##2.Graficar la serie temporal de ambas estaciones
###########################################################################
#Transformar data.frame a serie temporal
#costa1
ts_costa1 = as.POSIXct(paste(costa1a$year, costa1a$month, "01",sep="-"), format = "%Y-%b-%d") # crear serie temporal
z_costa = zoo(costa1a$rain,ts_costa1) #serie temporal

#costa2
ts_costa2 = as.POSIXct(paste(costa2a$year, costa2a$month, "01",sep="-"), format = "%Y-%b-%d") # crear serie temporal
z_costa2 = zoo(costa2a$rain,ts_costa2) #serie temporal

#unir costa1 y costa2
ts_dummy = seq(ts_costa1[1],ts_costa1[length(ts_costa1)],by="month")
z_dummy = zoo(rep(NA,length(ts_dummy)),ts_dummy) #zoo vacio
z_merge = merge(z_dummy,z_costa,z_costa2)

df_merge = data.frame(index(z_merge),coredata(z_merge))
df_merge = df_merge[,-2]
colnames(df_merge) = c("date","costa1","costa2")

#plot unido costa y costa2
melt_month = melt(df_merge, id="date")
uni_month = ggplot(melt_month, aes(x=date, y=value, color=variable))+
  geom_step()+
  theme_bw() +
  labs (title="Lluvia  mensual 1947-2012", x= NULL, y="Lluvia [mm/mes]")+
  theme(plot.title = element_text(size = 20), legend.position = "bottom",
        legend.text = element_text(size=14), legend.title = element_blank(),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))

png(file='series_temporales_costa2023A.png', height=400, width=800)
uni_month
dev.off()  

###########################################################################
##3. Determinar la homogeneidad de los datos de las estaciones mediante doble masas.
###########################################################################
#curvas de masa
cm_merge = df_merge
compl= which(complete.cases(df_merge$costa1,df_merge$costa2)) #datos comunes
cm_merge2 = cm_merge[compl,]  #selecciones solo periodos comunes
cm_merge2$ccosta1 = cumsum(cm_merge2$costa1)
cm_merge2$ccosta2 = cumsum(cm_merge2$costa2)

#pendiente curva doble masa (Ma)
slp = cm_merge2$ccosta2[length(cm_merge2$ccosta2)]/cm_merge2$ccosta1[length(cm_merge2$ccosta1)]

#plot doble masas costa 
cdm_month = ggplot(cm_merge2, aes(ccosta1, ccosta2))+ #x = costa1, y=costa2
  geom_line()+
  theme_bw() +
  scale_x_continuous(breaks=c(0,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000),
                     label=c("0","10K","20K","30K","40K","50K","60K","70K","80K","90K","100K"))+
  labs (title="Curva doble masas", x= "costa1 - P mensual acumuada [mm]", y= "costa2 - P mensual acumuada [mm]")+
  theme(plot.title = element_text(size = 20), legend.position = "bottom",
        legend.text = element_text(size=14), legend.title = element_blank(),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))+
  geom_abline(slope=slp, intercept=-15, color="red", linetype ="dashed")

png(file='curva_doble_masas_costa2023A.png', height=400, width=600)
cdm_month
dev.off() 

###########################################################################
##4. Correción de los datos de las estaciones (doble masas)
###########################################################################
cor_costa = which(cm_merge2$ccosta1>42500 & cm_merge2$ccosta1<75000)
#Mo
slp2 = (cm_merge2$ccosta2[cor_costa[length(cor_costa)]]-cm_merge2$ccosta2[cor_costa[1]])/(cm_merge2$ccosta1[cor_costa[length(cor_costa)]]-cm_merge2$ccosta1[cor_costa[1]])

cm_merge2$costa1cor =cm_merge2$costa1
cm_merge2$costa1cor[cor_costa] = cm_merge2$costa1[cor_costa]*(slp/slp2)
cm_merge2$costa2cor =cm_merge2$costa2
cm_merge2$costa2cor[cor_costa] = cm_merge2$costa2[cor_costa]*(slp/slp2)

cm_merge2$ccosta1cor = cumsum(cm_merge2$costa1cor)
cm_merge2$ccosta2cor = cumsum(cm_merge2$costa2cor)

#plot doble masas costa 
cdm_month_cor = ggplot(cm_merge2, aes(ccosta1cor, ccosta2cor))+ #x = costa1, y=costa2
  geom_line()+
  theme_bw() +
  scale_x_continuous(breaks=c(0,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000),
                     label=c("0","10K","20K","30K","40K","50K","60K","70K","80K","90K","100K"))+
  labs (title="Curva doble masas - Corregida", x= "costa1 - P mensual acumuada [mm]", y= "costa2 - P mensual acumuada [mm]")+
  theme(plot.title = element_text(size = 20), legend.position = "bottom",
        legend.text = element_text(size=14), legend.title = element_blank(),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))+
  geom_abline(slope=slp, intercept=-15, color="red", linetype ="dashed")

png(file='curva_doble_masas_costa2023A_cor.png', height=400, width=600)
cdm_month_cor
dev.off() 

###########################################################################
##5. Crear nueva serie corregida
###########################################################################
df_merge$costa1cor= df_merge$costa1
df_merge$costa2cor= df_merge$costa2

dates_cor = cm_merge2$date[cor_costa]

index=NA
for (i in 1:length(dates_cor)) {
  ind =  which(df_merge$date == dates_cor[i])
  index = c(index, ind)
}
index = index[-1]

df_merge$costa1cor[index] = cm_merge2$costa1cor[cor_costa] #series corregidas
df_merge$costa2cor[index] = cm_merge2$costa2cor[cor_costa] #series corregidas

###########################################################################
##6. Encontrar el coeficiente de correlación (r) y coeficiente determinación (r2)
###########################################################################
hist(df_merge$costa1cor)
hist(df_merge$costa2cor) 
print(paste("Coef. Correlación= ",
            round(cor(cm_merge2$costa1cor,cm_merge2$costa2cor,use="complete.obs", method = "pearson"),2),
            "Coef. Determinación= ",
            round((cor(cm_merge2$costa1cor,cm_merge2$costa2,use="complete.obs", method = "pearson"))^2,2)
))

print(cor.test(cm_merge2$costa1cor,cm_merge2$costa2cor,use="complete.obs", method = "pearson"))

###########################################################################
##7.Realizar el relleno de datos de la estación costa1 mediante regresión lineal
###########################################################################
#inicio y fin series
inicio_costa1 = which(!is.na(df_merge$costa1))[1]
fin_costa1 = which(!is.na(df_merge$costa1))[length(which(!is.na(df_merge$costa1)))]

inicio_costa2 = which(!is.na(df_merge$costa2))[1]
fin_costa2= which(!is.na(df_merge$costa2))[length(which(!is.na(df_merge$costa2)))]

#% vacíos
p_v_costa1 = round(length(which(is.na(df_merge$costa1[inicio_costa1:fin_costa1])))/(length(df_merge$costa1[inicio_costa1:fin_costa1]))*100,1)
p_v_costa2 = round(length(which(is.na(df_merge$costa2[inicio_costa2:fin_costa2])))/(length(df_merge$costa2[inicio_costa2:fin_costa2]))*100,1)
print(paste("%vacíos Costa 1= ",p_v_costa1, " %vacíos Costa 2= ",p_v_costa2))

#var independiente (x) = costa2
#var dependiente (y)= costa1

#fit linear regression model
model <- lm(costa1cor ~ costa2cor, data=df_merge) #y ~ x ; v. ind ~ v.dep 
summary(model)

a = round(model$coefficients[1],3) # intersección
b = round(model$coefficients[2],3) # pendiente
r2= round(summary(model)$r.squared,2) #R2
eq1 = paste("y = ",a, "+ ", b, "* x",",  R2=", r2)
print(eq1)

#plot
lrg_month = ggplot(df_merge, aes(costa2cor,costa1cor))+ #x, y
  geom_point()+
  theme_bw() +
  geom_smooth(method = "lm", level=0.95)+
  geom_text(x = 500, y = 1350, label = eq1, parse = F)+
  labs (title="Regression Lineal", x= "costa1 [mm/mes]", y= "costa2 [mm/mes]")+
  ylim(0,1500)+ xlim(0,1500)+
  theme(plot.title = element_text(size = 20), legend.position = "bottom",
        legend.text = element_text(size=14), legend.title = element_blank(),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))

png(file='reg_lineal_costa2023A.png', height=500, width=600)
lrg_month
dev.off() 

#identificar fechas a rellenar de costa1
a_rell = which(is.na(df_merge$costa1cor[inicio_costa1:fin_costa1]))

#aplicar eq. regresion lineal para relleno
costa1rell = round(a + (b*df_merge$costa2cor[a_rell]),1)
df_merge$costa1rell = df_merge$costa1cor
df_merge$costa1rell[a_rell] = costa1rell

###########################################################################
##7.Graficar la serie temporal de ambas estaciones, luego del relleno.
###########################################################################
#plot unido costa y costa2 rellenada
melt_month_rell = melt(df_merge[,-c(2:4)], id="date")
rell_month = ggplot(melt_month_rell, aes(x=date, y=value, color=variable))+
  geom_step()+
  theme_bw() +
  labs (title="Lluvia  mensual 1949-2012 (relleno costa1)", x= NULL, y="Lluvia [mm/mes]")+
  theme(plot.title = element_text(size = 20), legend.position = "bottom",
        legend.text = element_text(size=14), legend.title = element_blank(),
        axis.title.x = element_text(size=16), axis.title.y = element_text(size=16),
        axis.text.y = element_text(size=14), axis.text.x = element_text(size=14))+
  scale_color_discrete(labels=c('costa1 (relleno)', 'costa2'))

png(file='series_temporales_rellena_costa1_2023A.png', height=400, width=800)
rell_month
dev.off()  
